import * as fs from 'node:fs';
import * as path from 'node:path';
import parseArgs from 'minimist';
import { transformSFC } from './index.mjs';
import '@vue/compiler-sfc';
import '@babel/types';
import 'ast-kit';
import 'magic-string-ast';

async function read(stream) {
  const chunks = [];
  for await (const chunk of stream) {
    chunks.push(chunk);
  }
  return Buffer.concat(chunks).toString("utf8");
}
async function main() {
  const args = parseArgs(process.argv.slice(2));
  const inputFile = args._[0];
  const inputPromise = inputFile ? fs.promises.readFile(inputFile, "utf8") : read(process.stdin);
  let config;
  if (args.config) {
    const { default: importedConfig } = await import(path.resolve(args.config));
    config = importedConfig;
  }
  const input = await inputPromise;
  const output = transformSFC(input, config);
  const outFile = args.print ? void 0 : args.out ?? inputFile;
  if (outFile) {
    await fs.promises.writeFile(outFile, output);
  } else {
    process.stdout.write(output + "\n");
  }
}
main();
